@@ -13,15 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import httplib
-import urllib
-import StringIO
+import sys
 import ssl
 import time
 
 from xml.etree import ElementTree as ET
 from pipes import quote as pquote
-import urlparse
 
 try:
     import simplejson as json
@@ -29,10 +26,21 @@ except:
     import json
 
 import libcloud
+
+
+from libcloud.utils.py3 import PY3
+from libcloud.utils.py3 import httplib
+from libcloud.utils.py3 import urlparse
+from libcloud.utils.py3 import urlencode
+from libcloud.utils.py3 import StringIO
+from libcloud.utils.py3 import b
+
+from libcloud.utils.misc import lowercase_keys
 from libcloud.common.types import LibcloudError, MalformedResponseError
 
 from libcloud.httplib_ssl import LibcloudHTTPSConnection
-from httplib import HTTPConnection as LibcloudHTTPConnection
+
+LibcloudHTTPConnection = httplib.HTTPConnection
 
 class Response(object):
     """
@@ -50,8 +58,15 @@ class Response(object):
 
     def __init__(self, response, connection):
         self.body = response.read().strip()
+
+        if PY3:
+            self.body = b(self.body).decode('utf-8')
+
         self.status = response.status
-        self.headers = dict(response.getheaders())
+
+        # http.client In Python 3 doesn't automatically lowercase the header
+        # names
+        self.headers = lowercase_keys(dict(response.getheaders()))
         self.error = response.reason
         self.connection = connection
 
@@ -160,7 +175,7 @@ class RawResponse(Response):
     @property
     def headers(self):
         if not self._headers:
-            self._headers = dict(self.response.getheaders())
+            self._headers = lowercase_keys(dict(self.response.getheaders()))
         return self._headers
 
     @property
@@ -198,7 +213,7 @@ class LoggingConnection():
             def __init__(self, s):
                 self.s = s
             def makefile(self, mode, foo):
-                return StringIO.StringIO(self.s)
+                return StringIO(self.s)
         rr = r
         if r.chunked:
             ht += "%x\r\n" % (len(body))
@@ -463,7 +478,7 @@ class Connection(object):
         params, headers = self.pre_connect_hook(params, headers)
 
         if params:
-            url = '?'.join((action, urllib.urlencode(params)))
+            url = '?'.join((action, urlencode(params)))
         else:
             url = action
 
@@ -476,14 +491,15 @@ class Connection(object):
             if raw:
                 self.connection.putrequest(method, url)
 
-                for key, value in headers.iteritems():
+                for key, value in list(headers.items()):
                     self.connection.putheader(key, str(value))
 
                 self.connection.endheaders()
             else:
                 self.connection.request(method=method, url=url, body=data,
                                         headers=headers)
-        except ssl.SSLError, e:
+        except ssl.SSLError:
+            e = sys.exc_info()[1]
             raise ssl.SSLError(str(e))
 
         if raw:
