@@ -18,7 +18,9 @@ Gandi driver base classes
 
 import time
 import hashlib
-import xmlrpclib
+
+from libcloud.utils.py3 import xmlrpclib
+from libcloud.utils.py3 import b
 
 import libcloud
 from libcloud.common.base import ConnectionKey
@@ -84,14 +86,16 @@ class GandiConnection(ConnectionKey):
 
         try:
             self._proxy = self.proxyCls(self._user_agent())
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault:
+            e = sys.exc_info()[1]
             raise GandiException(1000, e)
 
     def request(self, method, *args):
         """ Request xmlrpc method with given args"""
         try:
             return getattr(self._proxy, method)(self.key, *args)
-        except xmlrpclib.Fault, e:
+        except xmlrpclib.Fault:
+            e = sys.exc_info()[1]
             raise GandiException(1001, e)
 
 
@@ -124,7 +128,8 @@ class BaseGandiDriver(object):
                     return False
             except (KeyError, IndexError):
                 pass
-            except Exception, e:
+            except Exception:
+                e = sys.exc_info()[1]
                 raise GandiException(1002, e)
 
             time.sleep(check_interval)
@@ -160,8 +165,8 @@ class BaseObject(object):
         Note, for example, that this example will always produce the
         same UUID!
         """
-        return hashlib.sha1("%s:%s:%d" % \
-            (self.uuid_prefix, self.id, self.driver.type)).hexdigest()
+        return hashlib.sha1(b("%s:%s:%d" % \
+            (self.uuid_prefix, self.id, self.driver.type))).hexdigest()
 
 
 class IPAddress(BaseObject):
