@@ -16,8 +16,11 @@
 """
 Common utilities for OpenStack
 """
-import httplib
-from urllib2 import urlparse
+import sys
+
+from libcloud.utils.py3 import httplib
+from libcloud.utils.py3 import urlparse
+
 from libcloud.common.base import ConnectionUserAndKey, Response
 from libcloud.compute.types import LibcloudError, InvalidCredsError, MalformedResponseError
 
@@ -86,6 +89,9 @@ class OpenStackAuthConnection(ConnectionUserAndKey):
         self.urls = {}
         self.driver = self.parent_conn.driver
 
+    def morph_action_hook(self, action):
+        return action
+
     def add_default_headers(self, headers):
         headers['Accept'] = 'application/json'
         headers['Content-Type'] = 'application/json; charset=UTF-8'
@@ -96,13 +102,15 @@ class OpenStackAuthConnection(ConnectionUserAndKey):
             return self.authenticate_1_0()
         elif self.auth_version == "1.1":
             return self.authenticate_1_1()
-        elif self.auth_version == "2.0":
-            return self.authenticate_2_0()
+        elif self.auth_version == "2.0" or self.auth_version == "2.0_apikey":
+            return self.authenticate_2_0_with_apikey()
+        elif self.auth_version == "2.0_password":
+            return self.authenticate_2_0_with_password()
         else:
             raise LibcloudError('Unsupported Auth Version requested')
 
     def authenticate_1_0(self):
-        resp = self.request("/",
+        resp = self.request("/v1.0",
                     headers={
                         'X-Auth-User': self.user_id,
                         'X-Auth-Key': self.key,
@@ -130,7 +138,7 @@ class OpenStackAuthConnection(ConnectionUserAndKey):
 
     def authenticate_1_1(self):
         reqbody = json.dumps({'credentials': {'username': self.user_id, 'key': self.key}})
-        resp = self.request("/auth",
+        resp = self.request("/v1.1/auth",
                     data=reqbody,
                     headers={},
                     method='POST')
@@ -145,18 +153,30 @@ class OpenStackAuthConnection(ConnectionUserAndKey):
         else:
             try:
                 body = json.loads(resp.body)
-            except Exception, e:
+            except Exception:
+                e = sys.exc_info()[1]
                 raise MalformedResponseError('Failed to parse JSON', e)
             try:
                 self.auth_token = body['auth']['token']['id']
                 self.urls = body['auth']['serviceCatalog']
-            except KeyError, e:
+            except KeyError:
+                e = sys.exc_info()[1]
                 raise MalformedResponseError('Auth JSON response is missing required elements', e)
 
-    # 'keystone' - http://docs.openstack.org/api/openstack-identity-service/2.0/content/Identity-Service-Concepts-e1362.html
-    def authenticate_2_0(self):
+    def authenticate_2_0_with_apikey(self):
+        # API Key based authentication uses the RAX-KSKEY extension.
+        # https://github.com/openstack/keystone/tree/master/keystone/content/service
+        reqbody = json.dumps({'auth':{'RAX-KSKEY:apiKeyCredentials':{'username':self.user_id, 'apiKey':self.key}}})
+        return self.authenticate_2_0_with_body(reqbody)
+
+    def authenticate_2_0_with_password(self):
+        # Password based authentication is the only 'core' authentication method in Keystone at this time.
+        # 'keystone' - http://docs.openstack.org/api/openstack-identity-service/2.0/content/Identity-Service-Concepts-e1362.html
         reqbody = json.dumps({'auth':{'passwordCredentials':{'username':self.user_id, 'password':self.key}}})
-        resp = self.request('tokens/',
+        return self.authenticate_2_0_with_body(reqbody)
+
+    def authenticate_2_0_with_body(self, reqbody):
+        resp = self.request('/v2.0/tokens/',
                     data=reqbody,
                     headers={'Content-Type':'application/json'},
                     method='POST')
@@ -169,12 +189,17 @@ class OpenStackAuthConnection(ConnectionUserAndKey):
         else:
             try:
                 body = json.loads(resp.body)
-            except Exception, e:
+            except Exception:
+                e = sys.exc_info()[1]
                 raise MalformedResponseError('Failed to parse JSON', e)
+
             try:
-                self.auth_token = body['access']['token']['id']
-                self.urls = body['access']['serviceCatalog']
-            except KeyError, e:
+                access = body['access']
+                token = access['token']
+                self.auth_token = token['id']
+                self.urls = access['serviceCatalog']
+            except KeyError:
+                e = sys.exc_info()[1]
                 raise MalformedResponseError('Auth JSON response is missing required elements', e)
 
 class OpenStackBaseConnection(ConnectionUserAndKey):
@@ -265,9 +290,14 @@ class OpenStackBaseConnection(ConnectionUserAndKey):
 
             # TODO: Multi-region support
             if self._auth_version == '2.0':
+                self.tenant_ids = {}
+
                 for service in osa.urls:
-                    if service.get('type') == 'compute':
+                    service_type = service['type']
+                    if service_type == 'compute':
                         self.server_url = self._get_default_region(service.get('endpoints', []))
+
+                    self.tenant_ids[service_type] = service['endpoints'][0]['tenantId']
             elif self._auth_version in ['1.1', '1.0']:
                 self.server_url = self._get_default_region(osa.urls.get('cloudServers', []))
                 self.cdn_management_url = self._get_default_region(osa.urls.get('cloudFilesCDN', []))
