@@ -17,11 +17,15 @@
 Provides base classes for working with drivers
 """
 
+import sys
 import time
 import hashlib
 import os
 import socket
 import struct
+import binascii
+
+from libcloud.utils.py3 import b
 
 import libcloud.compute.ssh
 from libcloud.pricing import get_size_price
@@ -78,7 +82,7 @@ class Node(object):
     >>> from libcloud.compute.drivers.dummy import DummyNodeDriver
     >>> driver = DummyNodeDriver(0)
     >>> node = driver.create_node()
-    >>> node.public_ip[0]
+    >>> node.public_ips[0]
     '127.0.0.3'
     >>> node.name
     'dummy-3'
@@ -109,19 +113,37 @@ class Node(object):
 
     """
 
-    def __init__(self, id, name, state, public_ip, private_ip,
+    def __init__(self, id, name, state, public_ips, private_ips,
                  driver, size=None, image=None, extra=None):
         self.id = str(id) if id else None
         self.name = name
         self.state = state
-        self.public_ip = public_ip
-        self.private_ip = private_ip
+        self.public_ips = public_ips if public_ips else []
+        self.private_ips = private_ips if private_ips else []
         self.driver = driver
         self.uuid = self.get_uuid()
         self.size = size
         self.image = image
         self.extra = extra or {}
 
+    # Note: getters and setters bellow are here only for backward compatibility.
+    # They will be removed in the next release.
+
+    def _set_public_ips(self, value):
+        self.public_ips = value
+
+    def _get_public_ips(self):
+        return self.public_ips
+
+    def _set_private_ips(self, value):
+        self.private_ips = value
+
+    def _get_private_ips(self):
+        return self.private_ips
+
+    public_ip = property(fget=_get_public_ips, fset=_set_public_ips)
+    private_ip = property(fget=_get_private_ips, fset=_set_private_ips)
+
     def get_uuid(self):
         """Unique hash for this node
 
@@ -143,7 +165,7 @@ class Node(object):
         Note, for example, that this example will always produce the
         same UUID!
         """
-        return hashlib.sha1("%s:%d" % (self.id, self.driver.type)).hexdigest()
+        return hashlib.sha1(b("%s:%d" % (self.id, self.driver.type))).hexdigest()
 
     def reboot(self):
         """Reboot this node
@@ -189,9 +211,9 @@ class Node(object):
         return self.driver.destroy_node(self)
 
     def __repr__(self):
-        return (('<Node: uuid=%s, name=%s, state=%s, public_ip=%s, '
+        return (('<Node: uuid=%s, name=%s, state=%s, public_ips=%s, '
                  'provider=%s ...>')
-                % (self.uuid, self.name, self.state, self.public_ip,
+                % (self.uuid, self.name, self.state, self.public_ips,
                    self.driver.name))
 
 
@@ -484,7 +506,7 @@ class NodeDriver(BaseDriver):
         ...     try:
         ...         node = driver.deploy_node(deploy=msd)
         ...     except NotImplementedError:
-        ...         print "not implemented for dummy driver"
+        ...         print ("not implemented for dummy driver")
         >>> d()
         not implemented for dummy driver
 
@@ -507,7 +529,7 @@ class NodeDriver(BaseDriver):
                     'deploy_node not implemented for this driver')
 
             if 'auth' not in kwargs:
-                kwargs['auth'] = NodeAuthPassword(os.urandom(16).encode('hex'))
+                kwargs['auth'] = NodeAuthPassword(binascii.hexlify(os.urandom(16)))
 
             if 'ssh_key' not in kwargs:
                 password = kwargs['auth'].password
@@ -527,7 +549,7 @@ class NodeDriver(BaseDriver):
             ssh_timeout = kwargs.get('ssh_timeout', 10)
             ssh_key_file = kwargs.get('ssh_key', None)
 
-            ssh_client = SSHClient(hostname=node.public_ip[0],
+            ssh_client = SSHClient(hostname=node.public_ips[0],
                                    port=ssh_port, username=ssh_username,
                                    password=password,
                                    key=ssh_key_file,
@@ -542,7 +564,8 @@ class NodeDriver(BaseDriver):
                                         node=node,
                                         ssh_client=ssh_client,
                                         max_tries=3)
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             raise DeploymentError(node, e)
         return node
 
@@ -568,7 +591,7 @@ class NodeDriver(BaseDriver):
 
         while time.time() < end:
             nodes = self.list_nodes()
-            nodes = filter(lambda n: n.uuid == node.uuid, nodes)
+            nodes = list([n for n in nodes if n.uuid == node.uuid])
 
             if len(nodes) == 0:
                 raise LibcloudError(value=('Booted node[%s] ' % node
@@ -582,7 +605,7 @@ class NodeDriver(BaseDriver):
 
             node = nodes[0]
 
-            if (node.public_ip and node.state == NodeState.RUNNING):
+            if (node.public_ips and node.state == NodeState.RUNNING):
                 return node
             else:
                 time.sleep(wait_period)
