@@ -162,8 +162,8 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
                 id=vm['id'],
                 name=vm.get('displayname', None),
                 state=self.NODE_STATE_MAP[vm['state']],
-                public_ip=public_ips.get(vm['id'], {}).keys(),
-                private_ip=private_ips,
+                public_ips=public_ips.get(vm['id'], {}).keys(),
+                private_ips=private_ips,
                 driver=self,
                 extra={
                     'zoneid': vm['zoneid'],
@@ -227,8 +227,8 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
             id=node['id'],
             name=node['displayname'],
             state=self.NODE_STATE_MAP[node['state']],
-            public_ip=[],
-            private_ip=[],
+            public_ips=[],
+            private_ips=[],
             driver=self,
             extra={
                 'zoneid': location.id,
@@ -256,7 +256,7 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
         if result.get('success', '').lower() != 'true':
             return None
 
-        node.public_ip.append(addr['ipaddress'])
+        node.public_ips.append(addr['ipaddress'])
         addr = CloudStackAddress(node, addr['id'], addr['ipaddress'])
         node.extra['ip_addresses'].append(addr)
         return addr
@@ -265,7 +265,7 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
         "Release a public IP."
 
         node.extra['ip_addresses'].remove(address)
-        node.public_ip.remove(address.address)
+        node.public_ips.remove(address.address)
 
         self._async_request('disableStaticNat', ipaddressid=address.id)
         self._async_request('disassociateIpAddress', id=address.id)
