@@ -18,17 +18,20 @@ Amazon EC2 driver
 """
 from __future__ import with_statement
 
+import sys
 import base64
 import hmac
 import os
 import time
-import urllib
 import copy
 
 from hashlib import sha256
 from xml.etree import ElementTree as ET
 
-from libcloud.utils import fixxpath, findtext, findattr, findall
+from libcloud.utils.py3 import urlquote
+from libcloud.utils.py3 import b
+
+from libcloud.utils.xml import fixxpath, findtext, findattr, findall
 from libcloud.common.base import ConnectionUserAndKey
 from libcloud.common.aws import AWSBaseResponse
 from libcloud.common.types import (InvalidCredsError, MalformedResponseError,
@@ -131,9 +134,16 @@ EC2_INSTANCE_TYPES = {
         'disk': 1690,
         'bandwidth': None
     },
+    'cc2.8xlarge': {
+        'id': 'cc2.8xlarge',
+        'name': 'Cluster Compute Eight Extra Large Instance',
+        'ram': 63488,
+        'disk': 3370,
+        'bandwidth': None
     }
+}
 
-CLUSTER_INSTANCES_IDS = ['cg1.4xlarge', 'cc1.4xlarge']
+CLUSTER_INSTANCES_IDS = ['cg1.4xlarge', 'cc1.4xlarge', 'cc2.8xlarge']
 
 EC2_US_EAST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
 EC2_US_WEST_INSTANCE_TYPES = dict(EC2_INSTANCE_TYPES)
@@ -192,7 +202,7 @@ class EC2Response(AWSBaseResponse):
 
 class EC2Connection(ConnectionUserAndKey):
     """
-    Repersents a single connection to the EC2 Endpoint
+    Represents a single connection to the EC2 Endpoint
     """
 
     host = EC2_US_EAST_HOST
@@ -219,12 +229,12 @@ class EC2Connection(ConnectionUserAndKey):
                        HTTPRequestURI + "\n" +
                        CanonicalizedQueryString <from the preceding step>
         """
-        keys = params.keys()
+        keys = list(params.keys())
         keys.sort()
         pairs = []
         for key in keys:
-            pairs.append(urllib.quote(key, safe='') + '=' +
-                         urllib.quote(params[key], safe='-_~'))
+            pairs.append(urlquote(key, safe='') + '=' +
+                         urlquote(params[key], safe='-_~'))
 
         qs = '&'.join(pairs)
 
@@ -236,7 +246,7 @@ class EC2Connection(ConnectionUserAndKey):
         string_to_sign = '\n'.join(('GET', hostname, path, qs))
 
         b64_hmac = base64.b64encode(
-            hmac.new(secret_key, string_to_sign, digestmod=sha256).digest()
+            hmac.new(b(secret_key), b(string_to_sign), digestmod=sha256).digest()
         )
         return b64_hmac
 
@@ -298,6 +308,16 @@ class EC2NodeDriver(NodeDriver):
         tag = "{%s}%s" % (NAMESPACE, 'return')
         return element.findtext(tag) == 'true'
 
+    def _get_state_boolean(self, element):
+        """
+        Checks for the instances's state
+        """
+        state = findall(element=element,
+                        xpath='instancesSet/item/currentState/name',
+                        namespace=NAMESPACE)[0].text
+
+        return state in ('stopping', 'pending', 'starting')
+
     def _get_terminate_boolean(self, element):
         status = element.findtext(".//{%s}%s" % (NAMESPACE, 'name'))
         return any([term_status == status
@@ -340,8 +360,8 @@ class EC2NodeDriver(NodeDriver):
                         namespace=NAMESPACE),
             name=name,
             state=state,
-            public_ip=public_ips,
-            private_ip=private_ips,
+            public_ips=public_ips,
+            private_ips=private_ips,
             driver=self.connection.driver,
             extra={
                 'dns_name': findattr(element=element, xpath="dnsName",
@@ -431,8 +451,17 @@ class EC2NodeDriver(NodeDriver):
         )
         return n
 
-    def list_nodes(self):
+    def list_nodes(self, ex_node_ids=None):
+        """
+        @type node.id: C{list}
+        @param ex_node_ids: List of C{node.id}
+        This parameter is used to filter the list of
+        nodes that should be returned. Only the nodes
+        with the corresponding node ids will be returned.
+        """
         params = {'Action': 'DescribeInstances'}
+        if ex_node_ids:
+            params.update(self._pathlist('InstanceId', ex_node_ids))
         elem = self.connection.request(self.path, params=params).object
         nodes = []
         for rs in findall(element=elem, xpath='reservationSet/item',
@@ -446,7 +475,7 @@ class EC2NodeDriver(NodeDriver):
         nodes_elastic_ips_mappings = self.ex_describe_addresses(nodes)
         for node in nodes:
             ips = nodes_elastic_ips_mappings[node.id]
-            node.public_ip.extend(ips)
+            node.public_ips.extend(ips)
         return nodes
 
     def list_sizes(self, location=None):
@@ -458,7 +487,7 @@ class EC2NodeDriver(NodeDriver):
 
     def _get_sizes(self, include_cluser_instances=False):
         sizes = []
-        for key, values in self._instance_types.iteritems():
+        for key, values in self._instance_types.items():
             if not include_cluser_instances and\
                key in CLUSTER_INSTANCES_IDS:
                 continue
@@ -544,7 +573,7 @@ class EC2NodeDriver(NodeDriver):
             }
 
     def ex_describe_keypairs(self, name):
-        """Describes a keypiar by name
+        """Describes a keypair by name
 
         @note: This is a non-standard extension API, and only works for EC2.
 
@@ -601,7 +630,8 @@ class EC2NodeDriver(NodeDriver):
             results.append(
                 self.connection.request(self.path, params=params.copy()).object
             )
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             if e.args[0].find("InvalidPermission.Duplicate") == -1:
                 raise e
         params['IpProtocol'] = 'udp'
@@ -610,7 +640,8 @@ class EC2NodeDriver(NodeDriver):
             results.append(
                 self.connection.request(self.path, params=params.copy()).object
             )
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             if e.args[0].find("InvalidPermission.Duplicate") == -1:
                 raise e
 
@@ -620,7 +651,9 @@ class EC2NodeDriver(NodeDriver):
             results.append(
                 self.connection.request(self.path, params=params.copy()).object
             )
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
+
             if e.args[0].find("InvalidPermission.Duplicate") == -1:
                 raise e
         return results
@@ -749,6 +782,55 @@ class EC2NodeDriver(NodeDriver):
             'Filter.0.Value.0': node.id
         })
 
+    def ex_describe_all_addresses(self, only_allocated=False):
+        """
+        Return all the Elastic IP addresses for this account
+        optionally, return only the allocated addresses
+
+        @keyword  only_allocated: If true, return only those addresses
+                                  that are associated with an instance
+        @type     only_allocated: C{string}
+
+        @return   list list of elastic ips for this particular account.
+        """
+        params = {'Action': 'DescribeAddresses'}
+
+        result = self.connection.request(self.path,
+                                         params=params.copy()).object
+
+        # the list which we return
+        elastic_ip_addresses = []
+        for element in findall(element=result, xpath='addressesSet/item',
+                               namespace=NAMESPACE):
+            instance_id = findtext(element=element, xpath='instanceId',
+                                   namespace=NAMESPACE)
+
+            # if only allocated addresses are requested
+            if only_allocated and not instance_id:
+                continue
+
+            ip_address = findtext(element=element, xpath='publicIp',
+                                  namespace=NAMESPACE)
+
+            elastic_ip_addresses.append(ip_address)
+
+        return elastic_ip_addresses
+
+    def ex_associate_addresses(self, node, elastic_ip_address):
+        """
+        Associate an IP address with a particular node.
+
+        @type node: C{Node}
+        @param node: Node instance
+
+        """
+        params = {'Action': 'AssociateAddress'}
+
+        params.update(self._pathlist('InstanceId', [node.id]))
+        params.update({'PublicIp': elastic_ip_address})
+        res = self.connection.request(self.path, params=params).object
+        return self._get_boolean(res)
+
     def ex_describe_addresses(self, nodes):
         """
         Return Elastic IP addresses for all the nodes in the provided list.
@@ -936,6 +1018,26 @@ class EC2NodeDriver(NodeDriver):
         res = self.connection.request(self.path, params=params).object
         return self._get_boolean(res)
 
+    def ex_start_node(self, node):
+        """
+        Start the node by passing in the node object, does not work with
+        instance store backed instances
+        """
+        params = {'Action': 'StartInstances'}
+        params.update(self._pathlist('InstanceId', [node.id]))
+        res = self.connection.request(self.path, params=params).object
+        return self._get_state_boolean(res)
+
+    def ex_stop_node(self, node):
+        """
+        Stop the node by passing in the node object, does not work with
+        instance store backed instances
+        """
+        params = {'Action': 'StopInstances'}
+        params.update(self._pathlist('InstanceId', [node.id]))
+        res = self.connection.request(self.path, params=params).object
+        return self._get_state_boolean(res)
+
     def destroy_node(self, node):
         """
         Destroy node by passing in the node object
