@@ -19,6 +19,8 @@ import time
 import hashlib
 import copy
 
+from libcloud.utils.py3 import b
+
 from libcloud.common.types import InvalidCredsError, LibcloudError
 from libcloud.common.gogrid import GoGridConnection, BaseGoGridDriver
 from libcloud.compute.providers import Provider
@@ -83,7 +85,7 @@ class GoGridNode(Node):
     # so uuid of node should not change after add is completed
     def get_uuid(self):
         return hashlib.sha1(
-            "%s:%d" % (self.public_ip,self.driver.type)
+            b("%s:%d" % (self.public_ips,self.driver.type))
         ).hexdigest()
 
 class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
@@ -119,8 +121,8 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
         n = GoGridNode(id=id,
                  name=element['name'],
                  state=state,
-                 public_ip=[ip],
-                 private_ip=[],
+                 public_ips=[ip],
+                 private_ips=[],
                  extra={'ram': element.get('ram').get('name'),
                      'description': element.get('description', '')},
                  driver=self.connection.driver)
@@ -218,7 +220,7 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
 
     def list_sizes(self, location=None):
         sizes = []
-        for key, values in self._instance_types.iteritems():
+        for key, values in self._instance_types.items():
             attributes = copy.deepcopy(values)
             attributes.update({ 'price': self._get_size_price(size_id=key) })
             sizes.append(NodeSize(driver=self.connection.driver, **attributes))
@@ -279,7 +281,7 @@ class GoGridNodeDriver(BaseGoGridDriver, NodeDriver):
             nodes = self.list_nodes()
 
             for i in nodes:
-                if i.public_ip[0] == node.public_ip[0] and i.id is not None:
+                if i.public_ips[0] == node.public_ips[0] and i.id is not None:
                     return i
 
             waittime += interval
