@@ -28,17 +28,21 @@ Linode(R) is a registered trademark of Linode, LLC.
 """
 
 try:
-	import simplejson as json
+    import simplejson as json
 except ImportError:
-	import json
+    import json
 
 import itertools
 import os
+import binascii
 
 from copy import copy
 
+from libcloud.utils.py3 import PY3
+from libcloud.utils.py3 import u
+
 from libcloud.common.linode import (API_ROOT, LinodeException, LinodeConnection,
-	LINODE_PLAN_IDS)
+    LINODE_PLAN_IDS)
 from libcloud.compute.types import Provider, NodeState
 from libcloud.compute.base import NodeDriver, NodeSize, Node, NodeLocation
 from libcloud.compute.base import NodeAuthPassword, NodeAuthSSHKey
@@ -295,7 +299,7 @@ class LinodeNodeDriver(NodeDriver):
 
         # Step 2: linode.disk.createfromdistribution
         if not root:
-            root = os.urandom(8).encode('hex')
+            root = u(binascii.hexlifyos.urandom(8).encode('hex'))
         params = {
             "api_action":       "linode.disk.createfromdistribution",
             "LinodeID":         linode["id"],
@@ -438,8 +442,8 @@ class LinodeNodeDriver(NodeDriver):
         batch = []
         for o in objs:
             lid = o["LINODEID"]
-            nodes[lid] = n = Node(id=lid, name=o["LABEL"], public_ip=[],
-                private_ip=[], state=self.LINODE_STATES[o["STATUS"]],
+            nodes[lid] = n = Node(id=lid, name=o["LABEL"], public_ips=[],
+                private_ips=[], state=self.LINODE_STATES[o["STATUS"]],
                 driver=self.connection.driver)
             n.extra = copy(o)
             n.extra["PLANID"] = self._linode_plan_ids.get(o.get("TOTALRAM"))
@@ -448,7 +452,12 @@ class LinodeNodeDriver(NodeDriver):
         # Avoid batch limitation
         ip_answers = []
         args = [iter(batch)] * 25
-        izip_longest = getattr(itertools, 'izip_longest', _izip_longest)
+
+        if PY3:
+            izip_longest = itertools.zip_longest
+        else:
+            izip_longest = getattr(itertools, 'izip_longest', _izip_longest)
+
         for twenty_five in izip_longest(*args):
             twenty_five = [q for q in twenty_five if q]
             params = { "api_action": "batch",
@@ -462,10 +471,10 @@ class LinodeNodeDriver(NodeDriver):
         for ip_list in ip_answers:
             for ip in ip_list:
                 lid = ip["LINODEID"]
-                which = nodes[lid].public_ip if ip["ISPUBLIC"] == 1 else \
-                    nodes[lid].private_ip
+                which = nodes[lid].public_ips if ip["ISPUBLIC"] == 1 else \
+                    nodes[lid].private_ips
                 which.append(ip["IPADDRESS"])
-        return nodes.values()
+        return list(nodes.values())
 
     features = {"create_node": ["ssh_key", "password"]}
 
