@@ -16,10 +16,14 @@
 VMware vCloud driver.
 """
 import base64
-import httplib
+from libcloud.utils.py3 import httplib
+from libcloud.utils.py3 import urlparse
+from libcloud.utils.py3 import b
+
+urlparse = urlparse.urlparse
+
 import time
 
-from urlparse import urlparse
 from xml.etree import ElementTree as ET
 from xml.parsers.expat import ExpatError
 
@@ -38,9 +42,6 @@ VIRTUAL_MEMORY_VALS = [512] + [1024 * i for i in range(1,9)]
 
 DEFAULT_TASK_COMPLETION_TIMEOUT = 600
 
-def get_url_path(url):
-    return urlparse(url.strip()).path
-
 def fixxpath(root, xpath):
     """ElementTree wants namespaces in its xpaths, so here we add them."""
     namespace, root_tag = root.tag[1:].split("}", 1)
@@ -48,6 +49,9 @@ def fixxpath(root, xpath):
                             for e in xpath.split("/")])
     return fixed_xpath
 
+def get_url_path(url):
+    return urlparse(url.strip()).path
+
 class InstantiateVAppXML(object):
 
     def __init__(self, name, template, net_href, cpus, memory,
@@ -226,7 +230,7 @@ class VCloudConnection(ConnectionUserAndKey):
         return {
             'Authorization':
                 "Basic %s"
-                % base64.b64encode('%s:%s' % (self.user_id, self.key)),
+                % base64.b64encode(b('%s:%s' % (self.user_id, self.key))),
             'Content-Length': 0
         }
 
@@ -324,8 +328,8 @@ class VCloudNodeDriver(NodeDriver):
         node = Node(id=elm.get('href'),
                     name=name,
                     state=state,
-                    public_ip=public_ips,
-                    private_ip=private_ips,
+                    public_ips=public_ips,
+                    private_ips=private_ips,
                     driver=self.connection.driver)
 
         return node
@@ -525,7 +529,7 @@ class VCloudNodeDriver(NodeDriver):
             network = ''
 
         password = None
-        if kwargs.has_key('auth'):
+        if 'auth' in kwargs:
             auth = kwargs['auth']
             if isinstance(auth, NodeAuthPassword):
                 password = auth.password
@@ -585,7 +589,7 @@ class HostingComConnection(VCloudConnection):
         """hosting.com doesn't follow the standard vCloud authentication API"""
         return {
             'Authentication':
-                base64.b64encode('%s:%s' % (self.user_id, self.key)),
+                base64.b64encode(b('%s:%s' % (self.user_id, self.key))),
             'Content-Length': 0
         }
 
