@@ -18,7 +18,7 @@ __all__ = [
 ]
 
 
-from libcloud.utils import merge_valid_keys, get_new_obj
+from libcloud.utils.misc import merge_valid_keys, get_new_obj
 from libcloud.common.linode import (API_ROOT, LinodeException,
                                     LinodeConnection, LinodeResponse,
                                     LINODE_PLAN_IDS)
@@ -35,6 +35,8 @@ VALID_RECORD_EXTRA_PARAMS = ['Priority', 'Weight', 'Port', 'Protocol',
                              'TTL_sec']
 
 RECORD_TYPE_MAP = {
+    RecordType.NS: 'NS',
+    RecordType.MX: 'MX',
     RecordType.A: 'A',
     RecordType.AAAA: 'AAAA',
     RecordType.CNAME: 'CNAME',
@@ -71,7 +73,7 @@ class LinodeDNSDriver(DNSDriver):
     connectionCls = LinodeDNSConnection
 
     def list_record_types(self):
-        return RECORD_TYPE_MAP.keys()
+        return list(RECORD_TYPE_MAP.keys())
 
     def list_zones(self):
         params = {'api_action': 'domain.list'}
