@@ -13,17 +13,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import sys
 import base64
 import hashlib
 import hmac
-import httplib
 import time
-import urllib
-import urlparse
 
-from xml.etree import ElementTree
+from libcloud.utils.py3 import PY3
+from libcloud.utils.py3 import httplib
+from libcloud.utils.py3 import urlparse
+from libcloud.utils.py3 import urlencode
+from libcloud.utils.py3 import next
+from libcloud.utils.py3 import b
 
-from libcloud import utils
+if PY3:
+    from io import FileIO as file
+
+from libcloud.utils.files import read_in_chunks
 from libcloud.common.base import ConnectionUserAndKey, XmlResponse
 from libcloud.common.types import LazyList
 
@@ -83,11 +89,12 @@ class AtmosConnection(ConnectionUserAndKey):
             pathstring = pathstring[len(self.driver.path):]
         if params:
             if type(params) is dict:
-                params = params.items()
-            pathstring += '?' + urllib.urlencode(params)
+                params = list(params.items())
+            pathstring += '?' + urlencode(params)
         pathstring = pathstring.lower()
 
-        xhdrs = [(k, v) for k, v in headers.items() if k.startswith('x-emc-')]
+        xhdrs = [(k, v) for k, v in list(headers.items()) if
+                 k.startswith('x-emc-')]
         xhdrs.sort(key=lambda x: x[0])
 
         signature = [
@@ -100,8 +107,8 @@ class AtmosConnection(ConnectionUserAndKey):
         signature.extend([k + ':' + collapse(v) for k, v in xhdrs])
         signature = '\n'.join(signature)
         key = base64.b64decode(self.key)
-        signature = hmac.new(key, signature, hashlib.sha1).digest()
-        return base64.b64encode(signature)
+        signature = hmac.new(b(key), b(signature), hashlib.sha1).digest()
+        return base64.b64encode(b(signature))
 
 class AtmosDriver(StorageDriver):
     connectionCls = AtmosConnection
@@ -132,7 +139,8 @@ class AtmosDriver(StorageDriver):
         path = self._namespace_path(container_name + '/?metadata/system')
         try:
             result = self.connection.request(path)
-        except AtmosError, e:
+        except AtmosError:
+            e = sys.exc_info()[1]
             if e.code != 1003:
                 raise
             raise ContainerDoesNotExistError(e, self, container_name)
@@ -146,7 +154,8 @@ class AtmosDriver(StorageDriver):
         path = self._namespace_path(container_name + '/')
         try:
             result = self.connection.request(path, method='POST')
-        except AtmosError, e:
+        except AtmosError:
+            e = sys.exc_info()[1]
             if e.code != 1016:
                 raise
             raise ContainerAlreadyExistsError(e, self, container_name)
@@ -156,7 +165,8 @@ class AtmosDriver(StorageDriver):
         try:
             self.connection.request(self._namespace_path(container.name + '/'),
                                     method='DELETE')
-        except AtmosError, e:
+        except AtmosError:
+            e = sys.exc_info()[1]
             if e.code == 1003:
                 raise ContainerDoesNotExistError(e, self, container.name)
             elif e.code == 1023:
@@ -174,7 +184,8 @@ class AtmosDriver(StorageDriver):
 
             result = self.connection.request(path + '?metadata/user')
             user_meta = self._emc_meta(result)
-        except AtmosError, e:
+        except AtmosError:
+            e = sys.exc_info()[1]
             if e.code != 1003:
                 raise
             raise ObjectDoesNotExistError(e, self, object_name)
@@ -204,7 +215,8 @@ class AtmosDriver(StorageDriver):
 
         try:
             self.connection.request(request_path + '?metadata/system')
-        except AtmosError, e:
+        except AtmosError:
+            e = sys.exc_info()[1]
             if e.code != 1003:
                 raise
             method = 'POST'
@@ -225,7 +237,8 @@ class AtmosDriver(StorageDriver):
         else:
             meta_data = extra.get('meta_data', {})
         meta_data['md5'] = result_dict['data_hash']
-        user_meta = ', '.join([k + '=' + str(v) for k, v in meta_data.items()])
+        user_meta = ', '.join([k + '=' + str(v) for k, v in
+                               list(meta_data.items())])
         self.connection.request(request_path + '?metadata/user', method='POST',
                                 headers={'x-emc-meta': user_meta})
         result = self.connection.request(request_path + '?metadata/system')
@@ -245,10 +258,10 @@ class AtmosDriver(StorageDriver):
             iterator = iter(iterator)
 
         data_hash = hashlib.md5()
-        generator = utils.read_in_chunks(iterator, CHUNK_SIZE, True)
+        generator = read_in_chunks(iterator, CHUNK_SIZE, True)
         bytes_transferred = 0
         try:
-            chunk = generator.next()
+            chunk = next(generator)
         except StopIteration:
             chunk = ''
 
@@ -256,7 +269,7 @@ class AtmosDriver(StorageDriver):
 
         while True:
             end = bytes_transferred + len(chunk) - 1
-            data_hash.update(chunk)
+            data_hash.update(b(chunk))
             headers = {
                 'x-emc-meta': 'md5=' + data_hash.hexdigest(),
             }
@@ -267,7 +280,7 @@ class AtmosDriver(StorageDriver):
             bytes_transferred += len(chunk)
 
             try:
-                chunk = generator.next()
+                chunk = next(generator)
             except StopIteration:
                 break
             if len(chunk) == 0:
@@ -280,7 +293,8 @@ class AtmosDriver(StorageDriver):
         else:
             meta_data = extra.get('meta_data', {})
         meta_data['md5'] = data_hash
-        user_meta = ', '.join([k + '=' + str(v) for k, v in meta_data.items()])
+        user_meta = ', '.join([k + '=' + str(v) for k, v in
+                               list(meta_data.items())])
         self.connection.request(path + '?metadata/user', method='POST',
                                 headers={'x-emc-meta': user_meta})
 
@@ -315,7 +329,7 @@ class AtmosDriver(StorageDriver):
         path = self._namespace_path(obj.container.name + '/' + obj.name)
         response = self.connection.request(path, method='GET', raw=True)
 
-        return self._get_object(obj=obj, callback=utils.read_in_chunks,
+        return self._get_object(obj=obj, callback=read_in_chunks,
                                 response=response,
                                 callback_kwargs={
                                     'iterator': response.response,
@@ -327,7 +341,8 @@ class AtmosDriver(StorageDriver):
         path = self._namespace_path(obj.container.name + '/' + obj.name)
         try:
             self.connection.request(path, method='DELETE')
-        except AtmosError, e:
+        except AtmosError:
+            e = sys.exc_info()[1]
             if e.code != 1003:
                 raise
             raise ObjectDoesNotExistError(e, self, obj.name)
@@ -358,7 +373,7 @@ class AtmosDriver(StorageDriver):
         ]
         params.append(('signature', self._cdn_signature(path, params)))
 
-        params = urllib.urlencode(params)
+        params = urlencode(params)
         path = self.path + path
         return urlparse.urlunparse((protocol, self.host, path, '', params, ''))
 
