@@ -14,7 +14,11 @@
 # limitations under the License.
 import sys
 import unittest
-import httplib
+import types
+
+from libcloud.utils.py3 import httplib
+from libcloud.utils.py3 import method_type
+from libcloud.utils.py3 import u
 
 from libcloud.common.types import InvalidCredsError, MalformedResponseError
 from libcloud.compute.types import Provider
@@ -83,7 +87,8 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         OpenStackMockHttp.type = 'UNAUTHORIZED'
         try:
             self.driver = self.create_driver()
-        except InvalidCredsError, e:
+        except InvalidCredsError:
+            e = sys.exc_info()[1]
             self.assertEqual(True, isinstance(e, InvalidCredsError))
         else:
             self.fail('test should have thrown')
@@ -92,7 +97,8 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         OpenStackMockHttp.type = 'UNAUTHORIZED_MISSING_KEY'
         try:
             self.driver = self.create_driver()
-        except MalformedResponseError, e:
+        except MalformedResponseError:
+            e = sys.exc_info()[1]
             self.assertEqual(True, isinstance(e, MalformedResponseError))
         else:
             self.fail('test should have thrown')
@@ -101,7 +107,8 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         OpenStackMockHttp.type = 'INTERNAL_SERVER_ERROR'
         try:
             self.driver = self.create_driver()
-        except MalformedResponseError, e:
+        except MalformedResponseError:
+            e = sys.exc_info()[1]
             self.assertEqual(True, isinstance(e, MalformedResponseError))
         else:
             self.fail('test should have thrown')
@@ -118,8 +125,8 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         ret = self.driver.list_nodes()
         self.assertEqual(len(ret), 1)
         node = ret[0]
-        self.assertEqual('67.23.21.33', node.public_ip[0])
-        self.assertEqual('10.176.168.218', node.private_ip[0])
+        self.assertEqual('67.23.21.33', node.public_ips[0])
+        self.assertEqual('10.176.168.218', node.private_ips[0])
         self.assertEqual(node.extra.get('flavorId'), '1')
         self.assertEqual(node.extra.get('imageId'), '11')
         self.assertEqual(type(node.extra.get('metadata')), type(dict()))
@@ -144,8 +151,8 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
                          'created': '2009-11-29T20:22:09-06:00',
                          'updated': '2009-11-29T20:24:08-06:00',
                          'progress': '100'}}
-        for ret_idx, extra in expected.items():
-            for key, value in extra.items():
+        for ret_idx, extra in list(expected.items()):
+            for key, value in list(extra.items()):
                 self.assertEqual(ret[ret_idx].extra[key], value)
 
     def test_create_node(self):
@@ -183,14 +190,14 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual(node.extra.get('metadata'), metadata)
 
     def test_reboot_node(self):
-        node = Node(id=72258, name=None, state=None, public_ip=None,
-                    private_ip=None, driver=self.driver)
+        node = Node(id=72258, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
         ret = node.reboot()
         self.assertTrue(ret is True)
 
     def test_destroy_node(self):
-        node = Node(id=72258, name=None, state=None, public_ip=None,
-                    private_ip=None, driver=self.driver)
+        node = Node(id=72258, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
         ret = node.destroy()
         self.assertTrue(ret is True)
 
@@ -200,8 +207,8 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         self.assertTrue("absolute" in limits)
 
     def test_ex_save_image(self):
-        node = Node(id=444222, name=None, state=None, public_ip=None,
-                    private_ip=None, driver=self.driver)
+        node = Node(id=444222, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
         image = self.driver.ex_save_image(node, "imgtest")
         self.assertEqual(image.name, "imgtest")
         self.assertEqual(image.id, "12345")
@@ -267,20 +274,20 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEquals(True, ret)
 
     def test_ex_resize(self):
-        node = Node(id=444222, name=None, state=None, public_ip=None,
-                    private_ip=None, driver=self.driver)
+        node = Node(id=444222, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
         size = NodeSize(1, '256 slice', None, None, None, None,
                         driver=self.driver)
         self.assertTrue(self.driver.ex_resize(node=node, size=size))
 
     def test_ex_confirm_resize(self):
-        node = Node(id=444222, name=None, state=None, public_ip=None,
-                    private_ip=None, driver=self.driver)
+        node = Node(id=444222, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
         self.assertTrue(self.driver.ex_confirm_resize(node=node))
 
     def test_ex_revert_resize(self):
-        node = Node(id=444222, name=None, state=None, public_ip=None,
-                    private_ip=None, driver=self.driver)
+        node = Node(id=444222, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
         self.assertTrue(self.driver.ex_revert_resize(node=node))
 
     def test_list_sizes(self):
@@ -393,6 +400,7 @@ class OpenStackMockHttp(MockHttpTestCase):
     def _v1_0_slug_servers_EX_SHARED_IP_GROUP(self, method, url, body, headers):
         # test_create_node_ex_shared_ip_group
         # Verify that the body contains sharedIpGroupId XML element
+        body = u(body)
         self.assertTrue(body.find('sharedIpGroupId="12345"') != -1)
         body = self.fixtures.load('v1_slug_servers.xml')
         return (httplib.ACCEPTED, body, XML_HEADERS, httplib.responses[httplib.ACCEPTED])
@@ -440,6 +448,7 @@ class OpenStackMockHttp(MockHttpTestCase):
         return (httplib.ACCEPTED, "", {}, httplib.responses[httplib.ACCEPTED])
 
     def _v1_0_slug_servers_444222_action(self, method, url, body, headers):
+        body = u(body)
         if body.find('resize') != -1:
             # test_ex_resize_server
             return (httplib.ACCEPTED, "", headers, httplib.responses[httplib.NO_CONTENT])
@@ -456,19 +465,19 @@ class OpenStackMockHttp(MockHttpTestCase):
         headers.update(XML_HEADERS)
         return (httplib.OK, body, headers, httplib.responses[httplib.OK])
 
-    def _v1_1__auth(self, method, url, body, headers):
+    def _v1_1_auth(self, method, url, body, headers):
         body = self.auth_fixtures.load('_v1_1__auth.json')
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _v1_1__auth_UNAUTHORIZED(self, method, url, body, headers):
+    def _v1_1_auth_UNAUTHORIZED(self, method, url, body, headers):
         body = self.auth_fixtures.load('_v1_1__auth_unauthorized.json')
         return  (httplib.UNAUTHORIZED, body, self.json_content_headers, httplib.responses[httplib.UNAUTHORIZED])
 
-    def _v1_1__auth_UNAUTHORIZED_MISSING_KEY(self, method, url, body, headers):
+    def _v1_1_auth_UNAUTHORIZED_MISSING_KEY(self, method, url, body, headers):
         body = self.auth_fixtures.load('_v1_1__auth_mssing_token.json')
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _v1_1__auth_INTERNAL_SERVER_ERROR(self, method, url, body, headers):
+    def _v1_1_auth_INTERNAL_SERVER_ERROR(self, method, url, body, headers):
         return (httplib.INTERNAL_SERVER_ERROR, "<h1>500: Internal Server Error</h1>",  {'content-type': 'text/html'}, httplib.responses[httplib.INTERNAL_SERVER_ERROR])
 
 class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
@@ -499,10 +508,10 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         node = nodes[0]
 
         self.assertEqual('12065', node.id)
-        self.assertEqual('50.57.94.35', node.public_ip[0])
-        self.assertEqual('2001:4801:7808:52:16:3eff:fe47:788a', node.public_ip[1])
-        self.assertEqual('10.182.64.34', node.private_ip[0])
-        self.assertEqual('fec0:4801:7808:52:16:3eff:fe60:187d', node.private_ip[1])
+        self.assertEqual('50.57.94.35', node.public_ips[0])
+        self.assertEqual('2001:4801:7808:52:16:3eff:fe47:788a', node.public_ips[1])
+        self.assertEqual('10.182.64.34', node.private_ips[0])
+        self.assertEqual('fec0:4801:7808:52:16:3eff:fe60:187d', node.private_ips[1])
 
         self.assertEqual(node.extra.get('flavorId'), '2')
         self.assertEqual(node.extra.get('imageId'), '7')
@@ -549,9 +558,9 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         image = NodeImage(id=11, name='Ubuntu 8.10 (intrepid)', driver=self.driver)
         size = NodeSize(1, '256 slice', None, None, None, None, driver=self.driver)
         node = self.driver.create_node(name='racktest', image=image, size=size)
-        self.assertEqual(node.id, '52415800-8b69-11e0-9b19-734f565bc83b')
-        self.assertEqual(node.name, 'new-server-test')
-        self.assertEqual(node.extra['password'], 'GFf1j9aP')
+        self.assertEqual(node.id, '26f7fbee-8ce1-4c28-887a-bfe8e4bb10fe')
+        self.assertEqual(node.name, 'racktest')
+        self.assertEqual(node.extra['password'], 'racktestvJq7d3')
         self.assertEqual(node.extra['metadata']['My Server Name'], 'Apache1')
 
     def test_destroy_node(self):
@@ -563,14 +572,16 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
     def test_ex_set_password(self):
         try:
             self.driver.ex_set_password(self.node, 'New1&53jPass')
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             self.fail('An error was raised: ' + repr(e))
 
     def test_ex_rebuild(self):
         image = NodeImage(id=11, name='Ubuntu 8.10 (intrepid)', driver=self.driver)
         try:
             self.driver.ex_rebuild(self.node, image=image)
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             self.fail('An error was raised: ' + repr(e))
 
     def test_ex_resize(self):
@@ -578,19 +589,22 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
                         driver=self.driver)
         try:
             self.driver.ex_resize(self.node, size)
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             self.fail('An error was raised: ' + repr(e))
 
     def test_ex_confirm_resize(self):
         try:
             self.driver.ex_confirm_resize(self.node)
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             self.fail('An error was raised: ' + repr(e))
 
     def test_ex_revert_resize(self):
         try:
             self.driver.ex_revert_resize(self.node)
-        except Exception, e:
+        except Exception:
+            e = sys.exc_info()[1]
             self.fail('An error was raised: ' + repr(e))
 
     def test_ex_save_image(self):
@@ -604,7 +618,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
     def test_ex_set_server_name(self):
         old_node = Node(
             id='12064', name=None, state=None,
-            public_ip=None, private_ip=None, driver=self.driver,
+            public_ips=None, private_ips=None, driver=self.driver,
         )
         new_node = self.driver.ex_set_server_name(old_node, 'Bob')
         self.assertEqual('Bob', new_node.name)
@@ -612,7 +626,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
     def test_ex_set_metadata(self):
         old_node = Node(
             id='12063', name=None, state=None,
-            public_ip=None, private_ip=None, driver=self.driver,
+            public_ips=None, private_ips=None, driver=self.driver,
         )
         metadata = {'Image Version': '2.1', 'Server Label': 'Web Head 1'}
         returned_metadata = self.driver.ex_set_metadata(old_node, metadata)
@@ -621,7 +635,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
     def test_ex_get_metadata(self):
         node = Node(
             id='12063', name=None, state=None,
-            public_ip=None, private_ip=None, driver=self.driver,
+            public_ips=None, private_ips=None, driver=self.driver,
         )
 
         metadata = {'Image Version': '2.1', 'Server Label': 'Web Head 1'}
@@ -631,14 +645,14 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
     def test_ex_update_node(self):
         old_node = Node(
             id='12064',
-            name=None, state=None, public_ip=None, private_ip=None, driver=self.driver,
+            name=None, state=None, public_ips=None, private_ips=None, driver=self.driver,
         )
 
         new_node = self.driver.ex_update_node(old_node, name='Bob')
 
         self.assertTrue(new_node)
         self.assertEqual('Bob', new_node.name)
-        self.assertEqual('50.57.94.30', new_node.public_ip[0])
+        self.assertEqual('50.57.94.30', new_node.public_ips[0])
 
     def test_ex_get_node_details(self):
         node_id = '12064'
@@ -673,7 +687,7 @@ class OpenStack_1_1_FactoryMethodTests(OpenStack_1_1_Tests):
     driver_klass = OpenStack_1_1_NodeDriver
     driver_type = get_driver(Provider.OPENSTACK)
     driver_args = OPENSTACK_PARAMS + ('1.1',)
-
+    driver_kwargs = {'ex_force_auth_version': '1.0'}
 
 class OpenStack_1_1_MockHttp(MockHttpTestCase):
     fixtures = ComputeFileFixtures('openstack_v1.1')
@@ -684,7 +698,7 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
         body = self.auth_fixtures.load('_v2_0__auth.json')
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
-    def _v1_0_(self, method, url, body, headers):
+    def _v1_0(self, method, url, body, headers):
         headers = {
             'x-auth-token': 'FE011C19-CF86-4F87-BE5D-9229145D7A06',
             'x-server-management-url': 'https://api.example.com/v1.1/slug',
@@ -704,7 +718,21 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
     def _v1_1_slug_servers(self, method, url, body, headers):
-        body = self.fixtures.load('_servers.json')
+        if method == "POST":
+            body = self.fixtures.load('_servers_create.json')
+        elif method == "GET":
+            body = self.fixtures.load('_servers.json')
+        else:
+            raise NotImplementedError()
+
+        return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
+
+    def _v1_1_slug_servers_26f7fbee_8ce1_4c28_887a_bfe8e4bb10fe(self, method, url, body, headers):
+        if method == "GET":
+            body = self.fixtures.load('_servers_26f7fbee_8ce1_4c28_887a_bfe8e4bb10fe.json')
+        else:
+            raise NotImplementedError()
+
         return (httplib.OK, body, self.json_content_headers, httplib.responses[httplib.OK])
 
     def _v1_1_slug_servers_12065_action(self, method, url, body, headers):
@@ -770,16 +798,58 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
         else:
             raise NotImplementedError()
 
+class OpenStack_1_1_Auth_2_0_MockHttp(OpenStack_1_1_MockHttp):
+    fixtures = ComputeFileFixtures('openstack_v1.1')
+    auth_fixtures = OpenStackFixtures()
+    json_content_headers = {'content-type': 'application/json; charset=UTF-8'}
+
+    def __init__(self, *args, **kwargs):
+        super(OpenStack_1_1_Auth_2_0_MockHttp, self).__init__(*args, **kwargs)
+
+        # TODO Figure out why 1.1 tests are using some 1.0 endpoints
+        methods1 = OpenStackMockHttp.__dict__
+        methods2 = OpenStack_1_1_MockHttp.__dict__
+
+        names1 = [m for m in methods1 if m.find('_v1_0') == 0]
+        names2 = [m for m in methods2 if m.find('_v1_1') == 0]
+
+        for name in names1:
+            method = methods1[name]
+            new_name = name.replace('_v1_0_slug_', '_v1_0_1337_')
+            setattr(self, new_name, method_type(method, self,
+                OpenStack_1_1_Auth_2_0_MockHttp))
+
+        for name in names2:
+            method = methods2[name]
+            new_name = name.replace('_v1_1_slug_', '_v1_0_1337_')
+            setattr(self, new_name, method_type(method, self,
+                OpenStack_1_1_Auth_2_0_MockHttp))
+
+
 class OpenStack_1_1_Auth_2_0_Tests(OpenStack_1_1_Tests):
+    driver_args = OPENSTACK_PARAMS + ('1.1',)
     driver_kwargs = {'ex_force_auth_version': '2.0'}
 
     def setUp(self):
-        self.driver_klass.connectionCls.conn_classes = (OpenStack_1_1_MockHttp, OpenStack_1_1_MockHttp)
+        self.driver_klass.connectionCls.conn_classes = \
+                (OpenStack_1_1_Auth_2_0_MockHttp, OpenStack_1_1_Auth_2_0_MockHttp)
         self.driver_klass.connectionCls.auth_url = "https://auth.api.example.com/v2.0/"
         OpenStack_1_1_MockHttp.type = None
         self.driver = self.create_driver()
         clear_pricing_data()
         self.node = self.driver.list_nodes()[1]
 
+        server_url = 'https://servers.api.rackspacecloud.com/v1.0/1337'
+        auth_token = 'aaaaaaaaaaaa-bbb-cccccccccccccc'
+        tenant_compute = '1337'
+        tenant_object_store = 'MossoCloudFS_11111-111111111-1111111111-1111111'
+
+        self.assertEqual(self.driver.connection.server_url, server_url)
+        self.assertEqual(self.driver.connection.auth_token, auth_token)
+        self.assertEqual(self.driver.connection.tenant_ids,
+              {'compute': tenant_compute, 'object-store': tenant_object_store})
+
+
+
 if __name__ == '__main__':
     sys.exit(unittest.main())
